"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getOrFindByIdParams = exports.changesByIdAfter = exports.changesByIdBefore = void 0;
const feathers_hooks_common_1 = require("feathers-hooks-common");
const cloneDeep_1 = __importDefault(require("lodash/cloneDeep"));
const get_1 = __importDefault(require("lodash/get"));
const set_1 = __importDefault(require("lodash/set"));
const isEqual_1 = __importDefault(require("lodash/isEqual"));
const feathers_utils_1 = require("feathers-utils");
const defaultOptions = {
    skipHooks: false,
    params: undefined,
    name: "changesById",
    deleteParams: [],
    fetchBefore: false
};
const changesById = (cb, _options) => {
    const options = Object.assign({}, defaultOptions, _options);
    return (context) => __awaiter(void 0, void 0, void 0, function* () {
        if ((0, feathers_utils_1.shouldSkip)("checkMulti", context)) {
            return context;
        }
        const pathBefore = getPath(options.name, true);
        if (context.type === "before") {
            const changes = yield (0, exports.changesByIdBefore)(context, options);
            if (!changes) {
                return context;
            }
            (0, set_1.default)(context, pathBefore, changes);
        }
        else if (context.type === "after") {
            const itemsBefore = (0, get_1.default)(context, pathBefore);
            const changes = yield (0, exports.changesByIdAfter)(context, itemsBefore, cb, options);
            if (!changes) {
                return context;
            }
            (0, set_1.default)(context, getPath(options.name, false), changes);
        }
        return context;
    });
};
const updateMethods = ["update", "patch"];
const changesByIdBefore = (context, _options) => __awaiter(void 0, void 0, void 0, function* () {
    const options = Object.assign({}, defaultOptions, _options);
    let byId;
    if (context.method === "create" ||
        !options.fetchBefore) {
        byId = {};
    }
    else if (updateMethods.includes(context.method) ||
        context.method === "remove") {
        byId = yield getOrFindById(context, options.params, { skipHooks: options.skipHooks });
    }
    else {
        return;
    }
    return byId;
});
exports.changesByIdBefore = changesByIdBefore;
const changesByIdAfter = (context, itemsBefore, cb, _options) => __awaiter(void 0, void 0, void 0, function* () {
    if (!itemsBefore) {
        return;
    }
    const options = Object.assign({}, defaultOptions, _options);
    const items = yield resultById(context, options);
    if (!items) {
        return;
    }
    const itemsBeforeOrAfter = (context.method === "remove" && options.fetchBefore) ? itemsBefore : items;
    const changesById = Object.keys(itemsBeforeOrAfter).reduce((result, id) => {
        if (options.fetchBefore &&
            ((context.method !== "create" && !itemsBefore[id]) ||
                (context.method !== "remove" && !items[id]))) {
            throw new Error("Mismatch!");
            //return result;
        }
        const before = itemsBefore[id];
        const item = items[id];
        result[id] = {
            before: before,
            item: item
        };
        return result;
    }, {});
    if (cb && typeof cb === "function") {
        yield cb(changesById, context);
    }
    return changesById;
});
exports.changesByIdAfter = changesByIdAfter;
const getOrFindByIdParams = (context, makeParams, options) => __awaiter(void 0, void 0, void 0, function* () {
    var _a, _b, _c;
    if (context.id == null) {
        if (context.type === "before") {
            let params = (0, cloneDeep_1.default)(context.params);
            delete params.changesById;
            if (options === null || options === void 0 ? void 0 : options.deleteParams) {
                options.deleteParams.forEach(key => {
                    delete params[key];
                });
            }
            if ((_a = params.query) === null || _a === void 0 ? void 0 : _a.$select) {
                delete params.query.$select;
            }
            params = Object.assign({ paginate: false }, params);
            params = (typeof makeParams === "function")
                ? yield makeParams(params, context)
                : params;
            return params;
        }
        else {
            if (!makeParams && !((_b = context.params.query) === null || _b === void 0 ? void 0 : _b.$select)) {
                return;
            }
            const itemOrItems = (0, feathers_hooks_common_1.getItems)(context);
            const idField = getIdField(context);
            if (!itemOrItems) {
                return;
            }
            const fetchedItems = (Array.isArray(itemOrItems)) ? itemOrItems : [itemOrItems];
            const ids = fetchedItems.map(x => x && x[idField]);
            let params = {
                query: {
                    [idField]: { $in: ids }
                },
                paginate: false
            };
            params = (makeParams) ? yield makeParams(params, context) : params;
            return params;
        }
    }
    else {
        if (context.type === "after" &&
            !makeParams &&
            !((_c = context.params.query) === null || _c === void 0 ? void 0 : _c.$select)) {
            return;
        }
        const query = Object.assign({}, context.params.query);
        delete query.$select;
        let params = Object.assign({}, context.params, { query });
        delete params.changesById;
        if (options === null || options === void 0 ? void 0 : options.deleteParams) {
            options.deleteParams.forEach(key => {
                delete params[key];
            });
        }
        params = (typeof makeParams === "function")
            ? yield makeParams(params, context)
            : params;
        return params;
    }
});
exports.getOrFindByIdParams = getOrFindByIdParams;
const getOrFindById = (context, makeParams, _options) => __awaiter(void 0, void 0, void 0, function* () {
    const options = Object.assign({
        skipHooks: true,
        byId: true
    }, _options);
    let itemOrItems;
    const idField = getIdField(context);
    const params = yield (0, exports.getOrFindByIdParams)(context, makeParams, options);
    if (context.id == null) {
        const method = (options.skipHooks) ? "_find" : "find";
        itemOrItems = yield context.service[method](params);
        itemOrItems = itemOrItems && (itemOrItems.data || itemOrItems);
    }
    else {
        const method = (options.skipHooks) ? "_get" : "get";
        itemOrItems = yield context.service[method](context.id, params);
    }
    const items = (!itemOrItems)
        ? []
        : (Array.isArray(itemOrItems))
            ? itemOrItems
            : [itemOrItems];
    if (options.byId) {
        return items.reduce((byId, item) => {
            const id = item[idField];
            byId[id] = item;
            return byId;
        }, {});
    }
    else {
        return items;
    }
});
const resultById = (context, options) => __awaiter(void 0, void 0, void 0, function* () {
    if (!context.result) {
        return {};
    }
    let items;
    let params = yield (0, exports.getOrFindByIdParams)(context, options.params, options);
    if (params) {
        const contextParams = Object.assign({}, context.params);
        delete contextParams.changesById;
        if (options === null || options === void 0 ? void 0 : options.deleteParams) {
            options.deleteParams.forEach(key => {
                delete contextParams[key];
            });
        }
        if ((0, isEqual_1.default)(params, context.params)) {
            params = null;
        }
    }
    if (context.method === "remove" || !params) {
        let itemOrItems = context.result;
        itemOrItems = (Array.isArray(itemOrItems.data)) ? itemOrItems.data : itemOrItems;
        items = Array.isArray(itemOrItems) ? itemOrItems : [itemOrItems];
    }
    else {
        items = (yield getOrFindById(context, () => params, {
            skipHooks: options.skipHooks,
            byId: false
        }));
    }
    const idField = context.service.id;
    return items.reduce((byId, item) => {
        const id = item[idField];
        byId[id] = item;
        return byId;
    }, {});
});
const getIdField = (context) => {
    return context.service.options.id;
};
const getPath = (path, isBefore) => {
    if (isBefore) {
        if (typeof path === "string") {
            return `params.${path}.itemsBefore`;
        }
        else {
            return ["params", ...path, "itemsBefore"];
        }
    }
    else {
        if (typeof path === "string") {
            return `params.${path}`;
        }
        else {
            return ["params", ...path];
        }
    }
};
exports.default = changesById;
