"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.triggerAfter = exports.triggerBefore = void 0;
const feathers_hooks_common_1 = require("feathers-hooks-common");
const changesById_1 = require("./changesById");
const transformMustache_1 = __importDefault(require("../utils/transformMustache"));
const sift_1 = __importDefault(require("sift"));
const cloneDeep_1 = __importDefault(require("lodash/cloneDeep"));
const set_1 = __importDefault(require("lodash/set"));
const trigger = (options, callAction) => {
    if (!options) {
        throw new Error("You should define subscriptions");
    }
    return (context) => __awaiter(void 0, void 0, void 0, function* () {
        (0, feathers_hooks_common_1.checkContext)(context, null, ["create", "update", "patch", "remove"], "trigger");
        if (context.type === "before") {
            return yield (0, exports.triggerBefore)(context, options, callAction);
        }
        else if (context.type === "after") {
            return yield (0, exports.triggerAfter)(context, callAction);
        }
    });
};
const triggerBefore = (context, options, callAction) => __awaiter(void 0, void 0, void 0, function* () {
    var _a, _b;
    let subs = yield getSubscriptions(context, options);
    if (!(subs === null || subs === void 0 ? void 0 : subs.length)) {
        return context;
    }
    if (!Array.isArray(context.data)) {
        const result = [];
        yield Promise.all(subs.map((sub) => __awaiter(void 0, void 0, void 0, function* () {
            if (!sub.callAction && !callAction) {
                return;
            }
            sub.dataResolved = (typeof sub.conditionsData === "function")
                ? yield sub.conditionsData(context.data, context)
                : testCondition(context, context.data, sub.conditionsData);
            if (sub.dataResolved === false) {
                return;
            }
            result.push(sub);
        })));
        subs = result;
    }
    if (!(subs === null || subs === void 0 ? void 0 : subs.length)) {
        return context;
    }
    for (const sub of subs) {
        sub.paramsResolved = yield (0, changesById_1.getOrFindByIdParams)(context, sub.params, { deleteParams: ["trigger"] });
        sub.identifier = JSON.stringify(sub.paramsResolved);
        if ((_b = (_a = context.params.changesById) === null || _a === void 0 ? void 0 : _a[sub.identifier]) === null || _b === void 0 ? void 0 : _b.itemsBefore) {
            continue;
        }
        const before = yield (0, changesById_1.changesByIdBefore)(context, {
            skipHooks: false,
            params: () => sub.params ? sub.paramsResolved : null,
            deleteParams: ["trigger"],
            fetchBefore: sub.fetchBefore
        });
        (0, set_1.default)(context, ["params", "changesById", sub.identifier, "itemsBefore"], before);
    }
    setConfig(context, "subscriptions", subs);
    return context;
});
exports.triggerBefore = triggerBefore;
const triggerAfter = (context, callAction) => __awaiter(void 0, void 0, void 0, function* () {
    var _c, _d, _e;
    const subs = getConfig(context, "subscriptions");
    if (!(subs === null || subs === void 0 ? void 0 : subs.length)) {
        return context;
    }
    const now = new Date();
    const promises = [];
    for (const sub of subs) {
        const itemsBefore = (_d = (_c = context.params.changesById) === null || _c === void 0 ? void 0 : _c[sub.identifier]) === null || _d === void 0 ? void 0 : _d.itemsBefore;
        let changesById;
        if (itemsBefore) {
            changesById = yield (0, changesById_1.changesByIdAfter)(context, itemsBefore, null, {
                name: ["changesById", sub.identifier],
                params: sub.params,
                skipHooks: false,
                deleteParams: ["trigger"],
                fetchBefore: sub.fetchBefore
            });
            (0, set_1.default)(context, ["params", "changesById", sub.identifier], changesById);
        }
        changesById = (_e = context.params.changesById) === null || _e === void 0 ? void 0 : _e[sub.identifier];
        if (!changesById) {
            continue;
        }
        const changes = Object.values(changesById);
        for (const change of changes) {
            const { before } = change;
            const { item } = change;
            const changeForSub = change;
            const { conditionsResult, conditionsBefore } = sub;
            let mustacheView = {
                item,
                before,
                data: context.data,
                id: context.id,
                method: context.method,
                now,
                params: context.params,
                path: context.path,
                service: context.service,
                type: context.type,
                user: context.params,
            };
            if (sub.view) {
                if (typeof sub.view === "function") {
                    mustacheView = yield sub.view(mustacheView, { item: changeForSub, items: changes, subscription: sub, subscriptions: subs, context });
                }
                else {
                    mustacheView = Object.assign(mustacheView, sub.view);
                }
            }
            sub.resultResolved = (typeof conditionsResult === "function")
                ? yield conditionsResult({ item, before }, context)
                : testCondition(mustacheView, item, conditionsResult);
            if (!sub.resultResolved) {
                continue;
            }
            sub.beforeResolved = (typeof conditionsBefore === "function")
                ? yield conditionsBefore({ item, before }, context)
                : testCondition(mustacheView, before, conditionsBefore);
            if (!sub.beforeResolved) {
                continue;
            }
            const callActionFunction = (sub.callAction)
                ? sub.callAction
                : callAction;
            const promise = callActionFunction(changeForSub, { subscription: sub, items: changes, context });
            if (sub.isBlocking) {
                promises.push(promise);
            }
        }
    }
    yield Promise.all(promises);
    return context;
});
exports.triggerAfter = triggerAfter;
function setConfig(context, key, val) {
    context.params.trigger = context.params.trigger || {};
    context.params.trigger[key] = val;
}
function getConfig(context, key) {
    var _a;
    return (_a = context.params.trigger) === null || _a === void 0 ? void 0 : _a[key];
}
const defaultSubscription = {
    callAction: undefined,
    conditionsBefore: true,
    conditionsData: true,
    conditionsResult: true,
    dataResolved: undefined,
    beforeResolved: undefined,
    resultResolved: undefined,
    isBlocking: true,
    method: undefined,
    params: undefined,
    service: undefined,
    view: undefined,
    paramsResolved: undefined,
    identifier: null,
    fetchBefore: false
};
const getSubscriptions = (context, options) => __awaiter(void 0, void 0, void 0, function* () {
    const _subscriptions = (typeof options === "function")
        ? yield options(context)
        : options;
    if (!_subscriptions) {
        return;
    }
    const subscriptions = (Array.isArray(_subscriptions)) ? _subscriptions : [_subscriptions];
    const subscriptionsResolved = subscriptions.map(x => Object.assign({}, defaultSubscription, x));
    const { path, method } = context;
    return subscriptionsResolved.filter(sub => {
        if ((typeof sub.service === "string" && sub.service !== path) ||
            (Array.isArray(sub.service) && !sub.service.includes(path))) {
            return false;
        }
        if ((typeof sub.method === "string" && sub.method !== method) ||
            (Array.isArray(sub.method) && !sub.method.includes(method))) {
            return false;
        }
        return true;
    });
});
const testCondition = (mustacheView, item, conditions) => {
    if (conditions === true) {
        return true;
    }
    conditions = (0, cloneDeep_1.default)(conditions);
    const transformedConditions = (0, transformMustache_1.default)(conditions, mustacheView);
    return ((0, sift_1.default)(conditions)(item)) ? transformedConditions : false;
};
exports.default = trigger;
