import type { HookContext, Params } from "@feathersjs/feathers";
import type { Promisable } from "type-fest";
export declare type MethodName = "create" | "update" | "patch" | "remove";
export declare type Change<T = any> = {
    before: T;
    item: T;
};
export declare type ChangesById<T = any> = {
    [key: string]: Change<T>;
    [key: number]: Change<T>;
};
export declare type ManipulateParams = (params: Params, context: HookContext) => (Promisable<Params>);
export interface HookChangesByIdOptions {
    skipHooks: boolean;
    params?: ManipulateParams;
    deleteParams?: string[];
    name?: string | string[];
    fetchBefore?: boolean;
}
interface ViewContext<T = any> {
    item: Change<T>;
    subscription: Subscription;
    subscriptions: Subscription[];
    items: Change<T>[];
    context: HookContext;
}
export declare type CallActionOptions<T = any> = {
    subscription?: SubscriptionResolved;
    items?: Change<T>[];
    context?: HookContext;
};
export declare type CallAction<T = any> = (item: Change<T>, options?: CallActionOptions<T>) => (Promisable<void>);
export declare type HookTriggerOptions = Subscription | Subscription[] | ((context: HookContext) => Promisable<Subscription | Subscription[]>);
export declare type TransformView<T = any> = undefined | ((view: Record<string, any>, viewContext: ViewContext<T>) => Promisable<Record<string, any>>) | Record<string, any>;
export declare type Condition = true | Record<string, any> | ((item: any, context: HookContext) => Promisable<boolean>);
export interface Subscription {
    service?: string | string[];
    method?: string | string[];
    conditionsData?: Condition;
    conditionsResult?: Condition;
    conditionsBefore?: Condition;
    view?: TransformView;
    params?: ManipulateParams;
    isBlocking?: boolean;
    callAction?: CallAction;
    fetchBefore?: boolean;
    [key: string]: any;
    [key: number]: any;
}
export interface SubscriptionResolved extends Subscription {
    dataResolved: boolean | Record<string, any>;
    resultResolved: boolean | Record<string, any>;
    beforeResolved: boolean | Record<string, any>;
    paramsResolved: Params;
    identifier: string;
}
export {};
